;;;
;;; INIT
;;;
;;; Init Vectors + Ram
;;;

;;;
;;; Shadow ROM : the ROM is mapped at address 00000 at reset.
;;; 	
	ORG	ROM_LOW

	DC.L	REGA7			; supervisor stack
	DC.L	START			; program counter


;;;
;;; MONITOR code starts here
;;; 

	ORG	ROM_LOW+8

FIRST
V2	BRA	TRACE

	DC.B	'SVN rev   1039',0,0

;;;
;;; Initialize monitor working storage
;;;
INITHRAM
	LEA	BEGHRAM,A0		; start of work ram
	MOVE.L	#(ENDHRAM-BEGHRAM),D0	; number of bytes to clear
	CLR.L	D1
INIT	MOVE.B	D1,(A0)+		; clear one byte
	SUBQ.L	#1,D0
	BNE	INIT

	ADDR2MEM BERRMSG,AV2 		; install bus trap handler
	ADDR2MEM ADDRMSG,AV3 		; install address trap handler

	RTS


;;;***************************************************
;;; SPECIAL HANDLING FOR BUS ERROR AND ADDRESS ERROR *
;;;***************************************************

BERRMSG  MOVE.L  #'BUS ',$30

         BRA.S   VECTBE

ADDRMSG  MOVE.L  #'ADDR',$30


VECTBE   MOVE.L  (A7)+,BERRD
         MOVE.L  (A7)+,BERRD+4

         SAVEREGS
         BSR     FIXBUF
         MOVE.W  #$D0A,(A6)+
         MOVE.W  BERRD,D0
         BSR     PNT4HX         	; FORMAT FUNCTION CODE

         MOVE.B  #BLANK,(A6)+   	; SPACE
         MOVE.L  BERRD+2,D0
         BSR     PNT8HX         	; FORMAT ACCESS ADDRESS

         MOVE.B  #BLANK,(A6)+   	; SPACE
         MOVE.W  BERRD+6,D0
         BSR     PNT4HX         	; FORMAT INSTRUCTION REGISTER

         BSR     OUTPUT
         BRA     EVECT2         	; GO DISPLAY REGISTERS & PROMPT

MSG008   DC.B    'SYNTAX '

MSG008E  DC.B    'ERROR '

MSGEOT   DC.B    EOT

MSG021   DC.B    'WHAT',EOT

         DC.B    0              	; PAD BYTE


;;;  PRINT WHAT

WHAT     LEA     MSG021(PC),A5  	; PRINT 'WHAT' AND ENTER MACSBUG
WHAT93   BSR.S   FIXDATA
         CLR.W   TRACEON
MSG      BSR     OUT1CR
         BRA     MACSBUG


;;; PRINT ERROR

ERROR    LEA     MSG008E(PC),A5
         BRA     WHAT93

SYNTAX   LEA     MSG008(PC),A5  	; 'SYNTAX ERROR'
         BRA     WHAT93

;;; FORMAT PHYSICAL ADDRESS FROM (D0)
PPHY     LEA     MSG019(PC),A5
         BSR.S   FIXDATA
         BRA.S   P2PHY2

;;; FORMAT TWO PHYSICAL ADDRESSES FROM (A0) & (A1)
P2PHY    LEA     MSG019(PC),A5  	; 'PHYSICAL ADDRESS='
         BSR.S   FIXDATA
         MOVE.L  A0,D0
         BSR     PNT8HX         	; FORMAT ADDR1
         MOVE.B  #BLANK,(A6)+   	; SPACE FOR FORMATING
         MOVE.L  A1,D0
P2PHY2   BSR     PNT8HX         	; FORMAT ADDR2
         BSR     OUT1CR         	; DISPLAY IT
         RTS

MSG019   DC.B    'PHYSICAL ADDRESS=',EOT


	
;;;************************************************************
;;; -FIXDATA- SUBROUTINE...  MOVES MESSAGE POINTED TO BY (A5) *
;;;                          INTO "BUFFER". EOT, ($04), ENDS  *
;;;                          THE MOVE. AT COMPLETION (A5) IS  *
;;;                          POINTING AT THE BEGINNING, (A6)  *
;;;                          POINTS AT END.                   *
;;;************************************************************

FIXDATA  LEA     BUFFER,A6
FIXDADD  CMPI.B  #EOT,(A5)
         BEQ.S   FIXD2
         MOVE.B  (A5)+,(A6)+
         BRA     FIXDADD
FIXD2    LEA     BUFFER,A5
         RTS


;;;***********************************************************
;;; -FIXDCRLF- SUBROUTINE INSERTS A CARRIAGE RETURN AND LINE *
;;;                       FEED IN FRONT OF THE TEXT, THEN    *
;;;                       USES THE REMAINING PORTION OF THE  *
;;;                       FIXDATA.                           *
;;;***********************************************************

FIXDCRLF LEA     BUFFER,A6
         MOVE.W  #$0D0A,(A6)+   	; CR,LF
         BRA     FIXDADD


	
