;;;
;;; CRC32 asm routine.
;;;
;;; Source: http://ftp.urz.uni-heidelberg.de/ftp/pub/amiga/dev/asm/CRC32.lha
;;; 

;;; 
;;; CR command :  CR <from address> <to address>    ("to address" is included)
;;;
CRCMD
	LEA	SYNTAX(PC),A0
	BSR	FNEXTF
	BSR	GETA
	MOVE.L	D0,A3			; A3 = from begin address

	BSR	FNEXTF
	BSR	GETA
	MOVE.L	D0,A1			; A1 = from end address

	MOVE.L	A3,A0
	BSR	P2PHY			; print address (A0) & (A1)

	MOVE.L	A1,D2
	SUB.L	A3,D2
	BCS	SYNTAX			; end < begin
	ADDQ.L	#1,D2			; D2 = size

	BSR.S	CRC32BUF		; compute CRC

	BSR	FIXBUF			; show result
	MOVE.L	#'CRC3',(A6)+
	MOVE.L	#'2 = ',(A6)+ 
	BSR	PNT8HX
	BSR	OUT1CR
	BRA	MACSBUG

	
;;;
;;; compute CRC32 on a buffer
;;;
;;; - A0 : pointer to buffer
;;; - D2 : size
;;; - return CRC in D0.L
;;; 
;;;
;;; Uses : A3, A5, D1, D4, D7. 
;;;        D2 is destroyed.
;;; 
CRC32BUF
	ADDQ.L	#1,D2
	
	MOVEQ	#0,D7
	NOT.B	D7			; D7 = 0xff

	LEA.L	CRC32TAB,A3
	CLR.L	D4			; crc = 0
	NOT.L	D4			; crc = ~crc

	MOVE.L	A0,A5
	BRA.S	CRC32_01

	DS	0
CRC32_00
			; crc = table[(crc ^ *buf) & 0xff] ^ (crc >> 8)
	MOVEQ	#0,D1
	MOVE.B	(A5)+,D1
	MOVE.L	D4,D0
	EOR.L	D1,D4
	AND.L	D7,D4
	LSL.L	#2,D4
	MOVE.L	(A3,D4.L),D4
	LSR.L	#8,D0
	EOR.L	D0,D4
CRC32_01
	SUBQ.L	#1,D2
	BNE.S	CRC32_00

	MOVE.L	D4,D0
	NOT.L	D0
	RTS


;;;
;;; Update CRC32
;;;
;;; - current CRC32 in D6.L
;;; - byte to add in D0.B
;;;
CRC32
	MOVEM.L	D0-D1/A3,-(SP)
	MOVEQ	#0,D1
	MOVE.B	D0,D1			; D1 = *buf
	MOVE.L	D6,D0			; D0 = current crc
	EOR.L	D1,D0			; D0 = crc ^ *buf
	ANDI.L	#$FF,D0			; D0 = (crc ^ *buf) & 0xff
	LEA	CRC32TAB(PC),A3
	LSL.L	#2,D0
	MOVE.L	(A3,D0.L),D0		; D0 = table[D0]
	LSR.L	#8,D6
	EOR.L	D0,D6
	MOVEM.L	(SP)+,D0-D1/A3
	RTS


;;;
;;; Update CRC16
;;;
;;; - current CRC16 in D6.W
;;; - byte to add in D0.B
;;;
CRC16
	MOVEM.L	D2/A0,-(SP)
	LEA	CRC16TAB(PC),A0
	MOVE.W	D6,D2			; D2 = crc
	LSR.W	#8,D2			; D2.B = (crc >> 8) & 0xff
	LSL.L	#1,D2
	MOVE.W	(A0,D2.W),D2		; D2.W = crc16tab[(crc >> 8) & 0xff]
	LSL.W	#8,D6			; D6 = crc << 8
	EOR.B	D0,D6			; D6 = (crc << 8) ^ D0.B
	EOR.W	D2,D6			; D6 = D2 ^ D6 = new crc16
	MOVEM.L	(SP)+,D2/A0
	RTS
	

;;;
;;; compute CRC32 table
;;;
;; MAKECRC32TAB
;; 	MOVE.L	#$100,D4
;; 	LEA.L	$EDB88320,A3
;; 	LEA.L	CRC32TAB,A0
;; 	MOVEQ	#0,D1
;; MCRC32_00
;; 	MOVE.L	D1,D6
;; 	MOVEQ	#0,D5
;; MCRC32_01
;; 	MOVE.L	D6,D0
;; 	LSR.L	#1,D0
;; 	BTST	#0,D6
;; 	BEQ.S	MCRC32_02
;; 	MOVE.L	A3,D6
;; 	EOR.L	D0,D6
;; 	BRA.S	MCRC32_03
;; MCRC32_02
;; 	MOVE.L	D0,D6
;; MCRC32_03
;; 	ADDQ.L	#1,D5
;; 	MOVEQ	#8,D0
;; 	CMP.L	D0,D5
;; 	BCS.S	MCRC32_01
;; 	ADDQ.L	#1,D1
;; 	MOVE.L	D6,(A0)+
;; 	CMP.L	D4,D1
;; 	BCS.S	MCRC32_00
;; 	RTS

