;;;
;;; Definitions
;;;

;;; Memory map
;;;
;;; 00000 - 7FFFF  : 512K : RAM      (BS62LV4006)
;;; 
;;; 80000 - BFFFF  : 256K : FLASH    (AT49F002)
;;; 
;;; C0000 - DFFFF  : 128K : peripherals, DTACK autogenerated
;;; 
;;; 	C0000 - C3FFF  : DEV0_0 : LED
;;; 	C4000 - C7FFF  : DEV0_1 : SPO256-AL2
;;; 	C8000 - CBFFF  : DEV0_2
;;; 	CC000 - CFFFF  : DEV0_3
;;; 	D0000 - D3FFF  : DEV0_4
;;; 	D4000 - D7FFF  : DEV0_5
;;; 	D8000 - DBFFF  : DEV0_6
;;; 	DC000 - DFFFF  : DEV0_7 : M48T58 (8K NVRAM + real time clock)
;;;
;;; E0000 - EFFFF  : 64K : peripherals, must provide DTACK to GAL
;;;
;;; F0000 - FFFFF  : 64K : peripherals, synchronous bus (6800 compatible)
	

RAM_LOW		EQU	$00400
RAM_HIGH	EQU	$7FFFF
	
ROM_LOW		EQU	$80000
ROM_HIGH	EQU	$BFFFF

DEBUG_LED	EQU	$C0000

	
LCD_INS		EQU	$F0000
LCD_DATA	EQU	LCD_INS+1

M48T58		EQU	$DC000

PIC_BASE	EQU	$E0000
PIC_UART_STATUS	EQU	PIC_BASE
PIC_UART_DATA	EQU	PIC_BASE+1
PIC_DEV_SEL	EQU	PIC_BASE+2
PIC_DEV_DATA	EQU	PIC_BASE+3
PIC_INT_STATUS	EQU	PIC_BASE+7

;;;
;;; PIC DEV SELECT :  0 0 0 d2 d1 d0 r1 r0
;;; where
;;;    d[2-0] : 000 = timer0
;;;             001 = timer1
;;;
;;;    r[1-0] : 00 = 8 LSB
;;;             01 = 8 MSB
;;;             10 = status/control
;;;             11 = extra
PIC_TIMER0	EQU	(0<<2)
PIC_T0_LSB	EQU	PIC_TIMER0+0
PIC_T0_MSB	EQU	PIC_TIMER0+1
PIC_T0_CON	EQU	PIC_TIMER0+2
PIC_T0_PS	EQU	PIC_TIMER0+3
;;;    r[1-0] = 10  bit 0 : 1 = TIMER0_ON       0 = TIMER0_OFF
;;;                 bit 1 : 1 = auto reload     0 = no reload
;;;                 bit 2 : 1 = 8 bits          0 = 16 bits
PIC_T0_ON	EQU	1
PIC_T0_AUTO	EQU	2
PIC_T0_8BITS	EQU	4
;;;   r[1-0] = 11   bits[3-0] prescaler
;;;                 1 xxx = 1:1
;;;                 0 000 = 1:2
;;;                 0 001 = 1:4
;;;                 0 010 = 1:8
;;;                 0 011 = 1:16
;;;                 0 100 = 1:32
;;;                 0 101 = 1:64
;;;                 0 110 = 1:128
;;;                 0 111 = 1:256
PIC_T0_1_1	EQU	08
PIC_T0_1_2	EQU	00
PIC_T0_1_4	EQU	01
PIC_T0_1_8	EQU	02
PIC_T0_1_16	EQU	03
PIC_T0_1_32	EQU	04
PIC_T0_1_64	EQU	05
PIC_T0_1_128	EQU	06
PIC_T0_1_256	EQU	07


PIC_TIMER1	EQU	(1<<2)
PIC_T1_LSB	EQU	PIC_TIMER1+0
PIC_T1_MSB	EQU	PIC_TIMER1+1
PIC_T1_CON	EQU	PIC_TIMER1+2
PIC_T1_PS	EQU	PIC_TIMER1+3	
;;;    r[1-0] = 10  bit 0 : 1 = TIMER0_ON       0 = TIMER0_OFF
;;;                 bit 1 : 1 = auto reload     0 = no reload
PIC_T1_ON	EQU	1
PIC_T1_AUTO	EQU	2
;;;   r[1-0] = 11   bits[1-0] prescaler
;;;                 00 = 1:1
;;;                 01 = 1:2
;;;                 10 = 1:4
;;;                 11 = 1:8
PIC_T1_1_1	EQU	00
PIC_T1_1_2	EQU	01
PIC_T1_1_4	EQU	02
PIC_T1_1_8	EQU	03





;;; definitions (in alphabetical order)
BELL		EQU	$07
BLANK		EQU	$20
BKPOINT		EQU	$4AFB
BUFFSIZE	EQU	$80		; load.asm
BUFSIZE		EQU	80		; dcode68k.asm
CR		EQU	$0D
CTLD		EQU	$04
CTLH		EQU	$08
CTLW		EQU	$17
CTLX		EQU	$18
DEL		EQU	$7F
EOT		EQU	$04
LF		EQU	$0A
XON		EQU	$11

	
;;; dcode68k, code68k
OUTBSIZE 	EQU     80        	; BUFFER SIZE
FDATA    	EQU     10        	; OFFSET TO DATA
FOL      	EQU     31        	; OFFSET TO LABEL
FOC      	EQU     31        	; OFFSET TO OP-CODE (NO LABEL FIELD)
FOP      	EQU     39        	; OFFSET TO OPERAND (NO LABEL FIELD)

	
;;;
;;; MACROS
;;;

ADDR2MEM MACRO
	PEA	\1(PC)			; macro is equivalent to
	MOVE.L	(A7)+,\2		; MOVE.L \1,\2
	ENDM

SAVEREGS MACRO
	MOVE.L	A7,REGA7		; save stack pointer
	LEA	SV\@(PC),A7		; A7 = return address (for call to SAVE)
	MOVE.L	A7,TEMP			; TEMP = return address
	BRA	SAVE			; BSR without using stack !
SV\@	DS	0
	ENDM
