;;;
;;; ZMODEM
;;;

;;; D4 = error number, if any
;;; D5 = flags
F_MORE_DATA	EQU	0
F_ZACK_0	EQU	1
F_ZACK_1	EQU	2
	

;;;
;;; error code
;;;
Z_OK		EQU	$00
Z_ERR_CRC	EQU	$01
Z_ERR_TIMEOUT	EQU	$02
Z_ERR_LONGSP	EQU	$03
Z_ERR_GOTCAN	EQU	$04
Z_ERR_RETRY	EQU	$05
Z_ERR_BADHEX	EQU	$06
Z_ERR_BADCHAR	EQU	$07
	
;;;
;;; frametypes
;;;
ZPAD		EQU	'*'
ZBIN		EQU	'A'
ZHEX		EQU	'B'
ZBIN32		EQU	'C'

;;;
;;; header types
;;;
ZRQINIT		EQU	0	; request attention
ZRINIT		EQU	1	; attention header
ZSINIT		EQU	2
ZACK		EQU	3	; acknowledge request
ZFILE		EQU	4	; file name from sender
ZSKIP		EQU	5	; to sender: skip this file
ZNAK		EQU	6	; last packet was garbled
ZABORT		EQU	7	; abort batch transfers
ZFIN		EQU	8	; finish session
ZRPOS		EQU	9	; resume data transfer at this position
ZDATA		EQU	10	; data packets follow
ZEOF		EQU	11	; end of file
ZFERR		EQU	12	; fatal read or write error detected
ZCRC		EQU	13	; request for file CRC and response
ZCHALLENGE	EQU	14	; receiver's challenge
ZCOMPL		EQU	15	; request is complete
ZCAN		EQU	16	; other end canned session with CAN*5
ZFREECNT	EQU	17	; request for free bytes on filesystem
ZCOMMAND	EQU	18	; command from sending program
ZSTDERR		EQU	19	; output to standard error, data follow

;;;
;;; ZDLE sequence
;;;
ZCRCE		EQU	'h'	; crc next, frame ends, header packet follows
ZCRCG		EQU	'i'	; crc next, frame continues nonstop
ZCRCQ		EQU	'j'	; crc next, frame continues, ACK expected
ZCRCW		EQU	'k'	; crc next, ACK expected, end of frame
ZRUB0		EQU	'l'	; translate to rubout 0177 ($7F)
ZRUB1		EQU	'm'	; translate to rubout 0377 ($FF)

;;;
;;; zmodem properties
;;;
CANFC32		EQU	$20
CANFDX		EQU	$01
CANOVIO		EQU	$02

;;;
;;; special chars
;;; 
ZDLE		EQU	$18
CAN		EQU	$18
ZCNL		EQU	$02
ZCBIN		EQU	$01
ZCRECOV		EQU	$03

;;;
;;; parameters
;;;
Z_RETRY		EQU	10



;;;
;;; ZMODEM RECEIVE
;;; 
RZCMD
	LEA	SYNTAX(PC),A0
	BSR	FNEXTF
	BSR	GETA
	MOVE.L	D0,A3			; A3 = where to receive

	BSR	FIXBUF
	LEA	ZMSG(PC),A5
	BSR	FIXDCRLF
	BSR	OUTPUT

	BSR	z_receive
	
	BRA	MACSBUG


ZMSG	DC.B	'ZMODEM RECEIVE...',EOT

	DS	0
	


;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;;; ZModem receive
;;;
;;; - file is received in (A3)
;;; - return status in D0
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_receive
	MOVEM.L	D1-D7/A0-A6,-(SP)
	MOVE.L	A3,A6			; save A3
	MOVE.W	Z_RETRY,D3
zr_00
	BSR	z_get_header		; get zmodem header
	BNE	zr_error

	MOVE.B	Z_HEADER_TYPE,D0	; header type is ?
	CMPI.B	#ZCAN,D0		; ... ZCAN
	BEQ	zr_zcan			; yes. cancel transfer
	CMPI.B	#ZRQINIT,D0		; ... ZRQINIT
	BEQ	zr_zinit		; yes. send ZINIT
	CMPI.B	#ZFILE,D0		; ... ZFILE
	BEQ	zr_zfile		; yes. go receive the file
	CMPI.B	#ZFIN,D0		; ... ZFIN
	BEQ	zr_zfin
zr_retry
	DBRA	D3,zr_00		; something else : retry
	BSR	z_send_can		; too many retries, send CANcel
	MOVE.B	#Z_ERR_RETRY,D0		; status returned in D0
	BRA	zr_done
zr_zfile
	CLR.L	D5			; clear status flag
	BSR	z_get_data		; get file info
	BNE	zr_zfile_00
	MOVE.L	A6,A4			; A4 points to file info data, A3 = end of data
	BSR	z_get_filesize		; extract total size
	CLR.L	Z_GOOD_OFFSET		; start at the begining of the file
	BSR	z_progress
	MOVE.L	A6,A3			; restore A3
	BSR	z_receive_file		; and receive the file
	BRA	zr_zinit		; send ZRINIT afterwards
zr_zfile_00
	CMPI.B	#Z_ERR_CRC,D4		; something went wrong while getting file info
	BEQ	zr_zinit		; if bad crc, send ZRINIT again
	MOVE.L	D4,D0			; return error code in D0
	BRA	zr_done			; otherwise, abort.
zr_zfin
	MOVEQ	#ZFIN,D0		; reply that we are finished too
	BSR	z_send_hex_header
	BSR	z_getbyte		; get 'O'
	BSR	z_getbyte		; get 'O'
	CLR.L	D0
	BRA	zr_done
zr_zinit
	MOVEQ	#ZRINIT,D0
	BSR	z_send_hex_header
	BRA	zr_retry
zr_zcan
	MOVEQ	#Z_ERR_GOTCAN,D0
	BRA	zr_done
zr_error
	CMPI.B	#Z_ERR_TIMEOUT,D4
	BEQ	zr_zinit
	CMPI.B	#Z_ERR_CRC,D4
	BEQ	zr_zinit
	CMPI.B	#Z_ERR_BADHEX,D4
	BEQ	zr_zinit
	BSR	z_send_can
	MOVE.L	D4,D0
zr_done
	MOVEM.L	(SP)+,D1-D7/A0-A6
	RTS


;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;;; show progress to user
;;;
;;; total bytes = Z_FILESIZE     current position = Z_GOOD_OFFSET
;;;
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_progress
	TST.L	Z_FILESIZE		; if filesize is unknown
	BNE.S	zpro_00
	RTS				; then return directly
zpro_00
	MOVEM.L	D0-D2/A5-A6,-(SP)
	BSR	lcd_clear
	BSR	FIXBUF
	MOVE.L	Z_FILESIZE,D0
	SUB.L	Z_GOOD_OFFSET,D0
	BSR	HEX2DEC
	MOVE.B	#' ',(A6)+
	MOVE.B	#'(',(A6)+

	MOVE.L	Z_FILESIZE,D1
	DIVU.W	#100,D1			; D1 = total / 100
	MOVE.L	Z_GOOD_OFFSET,D2
	DIVU.W	D1,D2
	CLR.L	D0
	MOVE.W	D2,D0

	BSR	HEX2DEC
	MOVE.B	#'%',(A6)+
	MOVE.B	#')',(A6)+
	CLR.B	(A6)			; null terminated string for lcd_puts
	BSR	lcd_puts
	MOVEM.L	(SP)+,D0-D2/A5-A6	
	RTS

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;;; receive one file, by calling z_receive_data.
;;;
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 	
z_receive_file
	MOVEM.L	D3,-(SP)
	MOVE.W	#Z_RETRY,D3
zrf_00
	MOVEQ	#ZRPOS,D0
	BSR	z_send_hex_header	; request data at current offset
	BSR	z_get_header		; get answer
	BNE	zrf_err

	MOVE.B	Z_HEADER_TYPE,D0 	; header type is ?
	CMPI.B	#ZCAN,D0		; ... ZCAN
	BEQ	zrf_zcan
	CMPI.B	#ZDATA,D0		; ... ZDATA
	BNE	zrf_01
	BSR	z_receive_data		; yes... that's what we want !
	BRA	zrf_done
zrf_01
	DBRA	D3,zrf_00		; unknown header. retry.
	MOVEQ	#Z_ERR_RETRY,D4
	BRA	zrf_done
zrf_err
	CMPI.B	#Z_ERR_TIMEOUT,D4	; if timeout, retry
	BEQ	zrf_01
	CMPI.B	#Z_ERR_CRC,D4		; if crc error, retry
	BEQ	zrf_01
	CMPI.B	#Z_ERR_BADHEX,D4	; if badhex error, retry
	BEQ	zrf_01
zrf_done
	MOVEM.L	(SP)+,D3
	RTS
zrf_zcan
	MOVEQ	#Z_ERR_GOTCAN,D4
	BRA.S	zrf_done
	

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;;; receive the complete data, by calling get_data multiple times
;;;
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
z_receive_data
	MOVEM.L	D3,-(SP)
	BSET	#F_ZACK_0,D5
	BCLR	#F_ZACK_1,D5		; NeedZack = 1
	BSET	#F_MORE_DATA,D5		; more data = 1

zrd_init
	MOVE.W	#Z_RETRY,D3
zrd_00
	BTST	#F_MORE_DATA,D5		; need more data ?
	BEQ	zrd_10			; no...
	BSR	z_get_data		; yes, get another chunk
	BNE	zrd_error		; error ? go handle it
	BRA	zrd_init		; no.. continue
zrd_10
	BTST	#F_ZACK_1,D5		; NeedZack == 3 ?
	BEQ	zrd_11			; no
	BTST	#F_ZACK_0,D5
	BEQ	zrd_11			; no
	MOVEQ	#ZACK,D0		; yes. send ZACK header
	BSR	z_send_hex_header
	BCLR	#F_ZACK_1,D5		; NeedZack = 1
zrd_11
	BSR	z_get_header		; get next header
	BNE	zrd_error
	MOVE.B	Z_HEADER_TYPE,D0	; header type is ?
	CMPI.B	#ZCAN,D0		; ... ZCAN
	BEQ	zrd_zcan		; yes, stop now
	CMPI.B	#ZDATA,D0		; ... ZDATA
	BEQ	zrd_zdata		; yes, check position
	CMPI.B	#ZEOF,D0		; ... ZEOF
	BNE	zrd_00			; no. start again
zrd_zeof
	MOVE.L	Z_GOOD_OFFSET,D0
	SUB.L	Z_HEADER_DATA,D0		
	BNE	zrd_00			; no.. continue
	CLR.L	D4			; yes.. we got the data properly.
	BRA	zrd_done
zrd_zcan
	MOVEQ	#Z_ERR_GOTCAN,D4	; received cancel
	BRA	zrd_done
zrd_zdata
	MOVE.L	Z_GOOD_OFFSET,D0
	SUB.L	Z_HEADER_DATA,D0	
	BNE	zrd_00			; no... continue
	BSET	#F_MORE_DATA,D5		; yes. get next data chunk
	BRA	zrd_00
zrd_error
	BCLR	#F_MORE_DATA,D5	
	MOVEQ	#ZRPOS,D0		; send a ZRPOS header
	BSR	z_send_hex_header
	DBRA	D3,zrd_00		; and retry
	MOVEQ	#Z_ERR_RETRY,D4
zrd_done
	MOVEM.L	(SP)+,D3
	RTS	



;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;;; receive ZModem header
;;;
;;; - binary headers begin with ZPAD* ZDLE ZBIN <bin header>
;;; - hex headers begin with ZPAD* ZDLE ZHEX <hex header>
;;; - 5 consecutive CAN means abort.
;;; - Z_HEADER_TYPE is updated if success
;;;
;;; Uses : A1, A2, D1, D2
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_get_header
	BSR	z_getbyte
	BNE	zgh_err

	MOVEQ	#4,D2			; CAN count
zgh_00
	CMPI.B	#ZPAD,D0		; ZPAD ?
	BEQ	zgh_11
	CMPI.B	#CAN,D0
	BNE	zgh_02
	DBRA	D2,zgh_00		; got one CAN, loop
	MOVE.B	#ZCAN,Z_HEADER_TYPE
zgh_ok
	CLR.L	D4
zgh_err
	RTS
zgh_02
	MOVEQ	#4,D2			; reset count
zgh_03
	BSR	z_getbyte
	BNE	zgh_err
	BRA	zgh_00			; loop until ZPAD or 5 CAN (or error)
zgh_10
	CMPI.B	#ZPAD,D0		; got one or more ZPAD
	BNE	zgh_20
zgh_11
	BSR	z_getbyte
	BNE	zgh_err
	BRA	zgh_10			; loop while we get ZPAD
zgh_20	
	CMPI.B	#ZDLE,D0		; got ZDLE ?
	BNE	zgh_badchar

	BSR	z_getbyte
	BNE	zgh_err
	CMPI.B	#ZBIN,D0
	BEQ	z_get_bin_header
	CMPI.B	#ZHEX,D0
	BEQ	z_get_hex_header
				;; handle other header types here
zgh_badchar
	MOVEQ	#Z_ERR_BADCHAR,D4
	RTS
	

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
;;; receive bin header
;;;	( ZDLE 'A' ) F3 F2 F1 F0 CRC1 CRC2 CR LF XON
;;;
;;; - Use A1, A2, D1, D2
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 	
z_get_bin_header
	LEA	z_get_DLE(PC),A2
	BSR	z_get_xxx_header
	BRA	z_get_header_test_crc
	

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;;; receive hex header
;;;	( ZDLE 'B' ) F3-hex F2-hex F1-hex F0-hex CRC1-hex CRC2-hex CR LF XON
;;;
;;; header data are stored in reverse order, so that positions byte are big
;;; endian, and allow easy comparison.
;;;
;;; - Uses A1, A2, D1, D2
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_get_hex_header
	LEA	z_get_hexbyte(PC),A2
	BSR	z_get_xxx_header

	;; receive CR LF, discard (ignore timeout error)
	BSR	z_getbyte
	BSR	z_getbyte

	;; test CRC
z_get_header_test_crc	
	TST.L	D6
	BNE	z_badcrc
	CLR.L	D4			; no error
	RTS
	

;;; 
;;; receive header. Use function in (A2) to get char.
;;; 
z_get_xxx_header
	CLR.L	D6			; CRC = 0
	JSR	(A2)			; get next byte
	BNE	zgxh_err		; return if error
	MOVE.B	D0,Z_HEADER_TYPE	; remember header type
	BSR	CRC16			; update CRC

	;; receive header (4 bytes)
	LEA	Z_HEADER_DATA+4,A1
	MOVEQ	#3,D1			; 4 iterations
zgxh_0
	JSR	(A2)			; get next byte
	BNE	zgxh_err		; return if error
	BSR	CRC16			; update CRC
	MOVE.B	D0,-(A1)		; save byte in HEADER_DATA
	DBRA	D1,zgxh_0

	;; receive CRC (2 bytes)
	JSR	(A2)			; CRC1
	BNE	zgxh_err
	BSR	CRC16
	JSR	(A2)			; CRC2
	BNE	zgxh_err
	BSR	CRC16

	CLR.L	D4			; no error
	RTS

z_badcrc
	MOVEQ	#Z_ERR_CRC,D4
zgxh_err
	RTS


;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; get file size from received file info data
;;;
;;; A4 = pointer to data
;;; A3 = end of data
;;;
;;; First, we get the name of the file, followed by a null char. Then we
;;; get the file size (in decimal), followed by a space.
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_get_filesize
	MOVEM.L	D0/A5/A6,-(SP)
zgfs_00
	TST.B	(A4)+
	BEQ	zgfs_01			; if null, good !
	CMP.L	A4,A3
	BCC	zgfs_00			; loop until finding 0 or end of data
zgfs_err
	CLR.L	Z_FILESIZE		; didn't find 0..
zgfs_done
	MOVEM.L	(SP)+,D0/A5/A6
	RTS
zgfs_01
	MOVE.L	A4,A5			; A5 = ptr to chars
	MOVE.L	A3,A6			; A6 = end
	BSR	GETDECNU		; decode decimal number
	BNE	zgfs_err
	MOVE.L	D0,Z_FILESIZE
	BRA.S	zgfs_done
	

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;;; send hex header
;;;
;;; - header type in D0
;;;
;;; Uses : A1, D1, D2
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
z_send_hex_header
	MOVE.B	D0,Z_HEADER_TYPE
	CLR.L	D6			; CRC = 0
	LEA	Z_HEADER_DATA,A1

	;; adjust header data
	CMPI.B	#ZRINIT,D0
	BEQ	zshh_01			; type ZRINIT
	CMPI.B	#ZFIN,D0
	BEQ	zshh_00			; type ZFIN
	CMPI.B	#ZRQINIT,D0
	BEQ	zshh_00			; type ZRQINIT
	CMPI.B	#ZRPOS,D0
	BEQ	zshh_02			; type ZRPOS
	CMPI.B	#ZEOF,D0
	BEQ	zshh_02			; type ZEOF
	CMPI.B	#ZACK,D0
	BEQ	zshh_02			; type ZACK
	BRA.S	zshh_10
zshh_00
	CLR.L	(A1)			; header data = 0
	BRA.S	zshh_10	
zshh_01
	MOVE.L	#CANOVIO,(A1)		; header data = 00 00 00 CANOVIO
	BRA.S	zshh_10
zshh_02
	MOVE.B	Z_GOOD_OFFSET+0,3(A1)	; header data = offset
	MOVE.B	Z_GOOD_OFFSET+1,2(A1)
	MOVE.B	Z_GOOD_OFFSET+2,1(A1)
	MOVE.B	Z_GOOD_OFFSET+3,0(A1)
	
	;; send the header
zshh_10
	;; send ZPAD ZPAD ZDLE ZHEX type
	MOVEQ	#ZPAD,D0
	BSR	z_sendbyte		; ZPAD
	BSR	z_sendbyte		; ZPAD
	MOVEQ	#ZDLE,D0
	BSR	z_sendbyte		; ZDLE
	MOVEQ	#ZHEX,D0
	BSR	z_sendbyte		; ZHEX
	MOVE.B	Z_HEADER_TYPE,D0
	BSR	CRC16
	BSR	z_send_hexbyte		; header type

	;; send header data (32 bits)
	MOVEQ	#3,D2
zshh_11
	MOVE.B	(A1)+,D0		; get one byte from header data
	BSR	CRC16
	BSR	z_send_hexbyte
	DBRA	D2,zshh_11		; loop until 32 bits sent

	;; adjust CRC
	CLR.B	D0
	BSR	CRC16
	BSR	CRC16	

	;; send CRC (16 bits)
	MOVE.W	D6,D0	
	ROR.W	#8,D0			; 8 MSB
	BSR	z_send_hexbyte
	MOVE.B	D6,D0			; 8 LSB 
	BSR	z_send_hexbyte

	;; send CR LF
	MOVEQ	#CR,D0
	BSR	z_sendbyte
	MOVEQ	#LF,D0
	BSR	z_sendbyte

	;; eventually send XON
	MOVE.B	Z_HEADER_TYPE,D0
	CMPI.B	#ZFIN,D0		; if not ZFIN
	BEQ	zshh_done
	CMPI.B	#ZACK,D0		; and not ZACK
	BEQ	zshh_done
	MOVEQ	#XON,D0			; then send XON
	BSR	z_sendbyte
zshh_done
	RTS


;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; receive one data chunk, in (A3).
;;;
;;; Uses :
;;;	- D6 : crc
;;;     - D5 : flags
;;;     - D4 : error
;;;     - D0 : strach register
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_get_data
	;MOVEM.L	D3/A3,-(SP)	; XXX change back if we use a single 1k buffer
	MOVEM.L	D3,-(SP)	
	CLR.L	D6			; CRC = 0
	CLR.L	D3			; nbytes = 0
zgd_00
	BSR	z_get_DLE		; get next byte
	BNE	zgd_err
	BTST	#8,D0			; got special ?
	BNE	zgd_10			; yes...

	MOVE.B	D0,(A3)+		; no.. save in buffer
	BSR	CRC16			; update CRC
	ADDQ	#1,D3			; nbytes++
	CMPI.L	#1024,D3
	BLE	zgd_00			; need more bytes

	MOVEQ	#Z_ERR_LONGSP,D4	; missed the special frame at the end of data
	BRA	zgd_done

zgd_10
	BSR	CRC16
	CMPI.B	#ZCRCE,D0		; got ZCRCE ?
	BEQ	zgd_11
	CMPI.B	#ZCRCW,D0		; got ZCRCW ?
	BNE	zgd_12
	BSET	#F_ZACK_1,D5		; NeedZack |= 2
zgd_11	BCLR	#F_MORE_DATA,D5
zgd_12
	BSR	z_get_DLE		; get CRC-1
	BNE	zgd_err
	BSR	CRC16

	BSR	z_get_DLE		; get CRC-2
	BNE	zgd_err
	BSR	CRC16
	TST.L	D6			; test CRC
	BNE	zgd_20			; not zero => bad
	ADD.L	D3,Z_GOOD_OFFSET	; good offset += nbytes
	BSR	z_progress
	CLR.L	D4			; no error
	BRA	zgd_done
zgd_20
	MOVEQ	#Z_ERR_CRC,D4
	BRA	zgd_done
	
zgd_err
	BCLR	#F_MORE_DATA,D5
	MOVEQ	#1,D0			; status.Z = 0
zgd_done
	MOVEM.L	(SP)+,D3
	RTS
	

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; receive one byte, DLE encoded
;;;
;;; - Value returned in D0.W
;;;      byte decoded in D0.B
;;;      special flags in D0[15-8]
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_get_DLE
	MOVEM.L	D1,-(SP)
zgdle_start
	BSR	z_getbyte		; get byte
	BNE	zgdle_done		; return if error
	MOVE.B	D0,D1			; save received char
	ANDI.B	#$60,D1			; quick check for non control char
	BNE	zgdle_ok
				;; check special chars
	CMPI.B	#ZDLE,D0		; ZDLE 
	BEQ	zgdle_00
	CMPI.B	#$11,D0			; ignore 0x11
	BEQ	zgdle_start
	CMPI.B	#$13,D0			; ignore 0x13
	BEQ	zgdle_start
	CMPI.B	#$91,D0			; ignore 0x91
	BEQ	zgdle_start
	CMPI.B	#$93,D0			; ignore 0x93
	BEQ	zgdle_start	
	BRA	zgdle_ok		; not special, return it
zgdle_00
	BSR	z_getbyte		; DLE encoded, get next byte
	BNE	zgdle_done
				;; check if we got CAN
	CMPI.B	#CAN,D0
	BNE	zgdle_10
				;; got one CAN. Need 4 extra CAN.
	MOVEQ	#3,D1	
zgdle_01
	BSR	z_getbyte
	BNE	zgdle_done
	CMPI.B	#CAN,D0			; loop while we get CAN
	BNE	zgdle_10
	DBRA	D1,zgdle_01
	MOVEQ	#Z_ERR_GOTCAN,D4	; 5 consecutive CAN, return
	BRA	zgdle_done
zgdle_10
	MOVE.B	D0,D1
	ANDI.B	#$60,D1
	CMPI.B	#$40,D1			; if ((c & 0x60) != 0x40)
	BNE	zgdle_20		; then flag special
	ANDI.B	#$BF,D0			; else bit 6 of D0 = 0
	BRA.S	zgdle_ok
zgdle_20
	ORI.W	#$0100,D0		; return ok, with special flag
zgdle_ok
	CLR.L	D4			; no error
zgdle_done
	MOVEM.L	(SP)+,D1
	RTS

	

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; send one byte, hex encoded
;;;
;;; - Value in D0.B
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_send_hexbyte
	MOVE.B	D0,-(SP)		; save byte
	ROXR.W	#4,D0
	BSR.S	zsh_00			; upper nibble
	MOVE.B	(SP)+,D0		; lower nibble
zsh_00
	ANDI.B	#$0F,D0			; isolate nibble
	ORI.B	#'0',D0			; 0 --> '0'
	CMPI.B	#$39,D0			; > '9' ?
	BLE.S	zsh_01			; no, send
	ADD.B	#$27,D0			; convert 10 -> 'a'
zsh_01
	BSR	z_sendbyte
	RTS

	
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; receive one byte, hex encoded
;;;
;;; - Value returned in D0.B
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_get_hexbyte
	MOVEM.L	D1,-(SP)
	BSR	z_get_hex		; receive upper nibble
	BNE.S	zghb_ret		; return if error
	MOVE.B	D0,D1
	ASL.B	#4,D1
	BSR	z_get_hex		; receive lower nibble
	BNE.S	zghb_ret
	OR.B	D1,D0			; assemble 2 nibbles into one byte
	CLR.L	D4
zghb_ret
	MOVEM.L	(SP)+,D1
	RTS

;;; receive one byte, hex encoded [0-9a-z]. Value returned in D0.B
z_get_hex
	BSR	z_getbyte
	BNE	zghb_err
	SUBI.B	#'0',D0			; < '0' ?
	BLT.S	zghb_badhex
	CMPI.B	#$9,D0			; > '9' ?
	BLE	zghb_ok
	SUBI.B	#$27,D0			; 'a' becomes 10
	CMPI.B	#$10,D0
	BCC.S	zghb_badhex
zghb_ok
	CLR.L	D4
zghb_err
	RTS
zghb_badhex
	MOVEQ	#Z_ERR_BADHEX,D4
	RTS
	

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; send CANcel (send 8 x CAN char)
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_send_can
	MOVE.L	D2,-(SP)
	MOVEQ	#7,D2
	MOVEQ	#CAN,D0
zsc_00	BSR	z_sendbyte
	MOVE.B	#50,D0
	BSR	delay_x_1ms
	DBRA	D2,zsc_00
	MOVE.L	(SP)+,D2
	RTS
	
	
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; receive one byte
;;;
;;; - char received in D0.B
;;; - update D4 if a timeout is detected
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_getbyte
	BSR	uart_getc_timeout
	BTST	#8,D0			; easy test to see if D0 == -1
	BEQ	zg_good
	MOVEQ	#Z_ERR_TIMEOUT,D4
	RTS
zg_good
	CLR.L	D4
	RTS

	
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; send one byte
;;;
;;; - char in D0.B (left intact)
;;; - update D4 if a timeout is detected
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
z_sendbyte
	BSR	uart_putc_timeout
	CLR.L	D4
	RTS
